import React, { useContext, useState, useEffect } from "react";
import { useNavigate } from "react-router-dom";
import moment from "moment";
import { useJobCore, jobStatus } from "@qltr/jobcore";

import Box from "@mui/material/Box";
import AppBar from "@mui/material/AppBar";
import Toolbar from "@mui/material/Toolbar";
import Typography from "@mui/material/Typography";

import Accordion from "@mui/material/Accordion";
import AccordionSummary from "@mui/material/AccordionSummary";
import Stack from "@mui/material/Stack";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import IconButton from "@mui/material/IconButton";
import ViewColumnIcon from "@mui/icons-material/ViewColumn";
import DoNotDisturbIcon from "@mui/icons-material/DoNotDisturb";

import Menu from "@mui/material/Menu";
import MenuItem from "@mui/material/MenuItem";
import ListItemText from "@mui/material/ListItemText";
import ListItemIcon from "@mui/material/ListItemIcon";
import MoreVertIcon from "@mui/icons-material/MoreVert";
import DeleteIcon from "@mui/icons-material/Delete";

function JobPipelineDisplay(props) {
  const { pipeline } = props;

  const {
    state: jobState,
    pipelineCancel,
    pipelineDestroy,
    selectedPipelineBranches,
    pipelineJobs,
    findPipelineJobByTestName,
    toJob,
    jobIcon,
  } = useJobCore();

  const nav = useNavigate();

  const [time, setTime] = useState(Date.now());
  const [anchorEl, setAnchorEl] = useState(null);
  const open = Boolean(anchorEl);
  const handleClick = (event) => setAnchorEl(event.currentTarget);
  const handleClose = () => setAnchorEl(null);

  const branches = selectedPipelineBranches(pipeline);
  const pipelineTriggers = branches.map(({ name }) =>
    pipeline.pendingTriggers.find(({ testName }) => testName === name)
  );

  useEffect(() => {
    const interval = setInterval(() => setTime(Date.now()), 1000);
    return () => clearInterval(interval);
  }, []);

  const selectJob = (testName) => () => {
    const pt = pipeline.pendingTriggers.find(
      ({ testName }) => testName === name
    );
    if (pt) return selectTimer(pt);
    const job = findPipelineJobByTestName(pipeline, testName);
    if (!job) return;
    toJob(job.jobId);
  };

  const selectTimer = (pt) => {
    console.log("Selected timer:", pt);
  };

  function cancelPipeline() {
    pipelineCancel(pipeline.id);
  }

  function deletePipeline() {
    pipelineDestroy(pipeline.id);
  }

  const menuSelect = (cb) => () => {
    handleClose();
    cb();
  };

  function pipelineActive() {
    return pipelineJobs(pipeline).find(
      (j) => j.status === jobStatus.ACTIVE || j.status === jobStatus.PENDING
    );
  }

  function boxIcon(name) {
    if (pipeline.pendingTriggers.find(({ testName }) => testName === name))
      return jobIcon({ status: jobStatus.TIMER });
    if (pipeline.isCanceled) return <DoNotDisturbIcon color="warning" />;
    const job = findPipelineJobByTestName(pipeline, name);
    if (!job) return <ViewColumnIcon color="secondary" />;
    return jobIcon(job);
  }

  function timerDisplay(name) {
    const pt = pipeline.pendingTriggers.find(
      ({ testName }) => testName === name
    );
    if (!pt) return;
    return moment(moment(pt.triggerAt).diff(moment())).format("mm:ss");
  }

  return (
    <Box>
      <AppBar
        position="fixed"
        sx={{
          backgroundColor: "white",
          boxShadow: "none",
          color: "black",
        }}
      >
        <Toolbar disableGutters />
        <Box sx={{ flexGrow: 1, margin: "0 10px" }}>
          <Toolbar disableGutters>
            <IconButton onClick={() => nav(-1)}>
              <ArrowBackIcon />
            </IconButton>
            <Typography
              variant="h6"
              component="span"
              sx={{
                ml: "auto",
                mr: "auto",
                textOverflow: "ellipsis",
                display: "block",
                whiteSpace: "nowrap",
                overflow: "hidden",
              }}
            >
              {pipeline.id}
            </Typography>
            <IconButton onClick={handleClick}>
              <MoreVertIcon />
            </IconButton>
          </Toolbar>
        </Box>
      </AppBar>
      <Toolbar disableGutters />
      {branches.map((track, i) => (
        <React.Fragment key={i}>
          <Typography variant="h6">{i + 1}</Typography>
          <Box>
            {track.map((test, j) => (
              <Accordion
                expanded={false}
                disableGutters={true}
                square
                key={j}
                onClick={selectJob(test.name)}
              >
                <AccordionSummary
                  style={{
                    backgroundColor: "rgba(0, 0, 0, .03)",
                    flexWrap: "wrap",
                  }}
                >
                  <Typography
                    component={"span"}
                    style={{ wordBreak: "break-word", margin: "auto 0" }}
                  >
                    {test.name}
                  </Typography>
                  <div style={{ marginLeft: "auto", display: "inline-flex" }}>
                    <Typography
                      component={"span"}
                      style={{ wordBreak: "break-word", margin: "auto 0" }}
                    >
                      {timerDisplay(test.name)}
                    </Typography>
                    <Stack sx={{ ml: "auto" }}>
                      <IconButton aria-label="retry" component="span">
                        {boxIcon(test.name)}
                      </IconButton>
                    </Stack>
                  </div>
                </AccordionSummary>
              </Accordion>
            ))}
          </Box>
        </React.Fragment>
      ))}

      <Menu anchorEl={anchorEl} open={open} onClose={handleClose}>
        {pipelineActive() ? (
          <MenuItem onClick={menuSelect(cancelPipeline)}>
            <ListItemIcon>
              <DoNotDisturbIcon fontSize="small" />
            </ListItemIcon>
            <ListItemText>Cancel</ListItemText>
          </MenuItem>
        ) : null}
        <MenuItem onClick={menuSelect(deletePipeline)}>
          <ListItemIcon>
            <DeleteIcon fontSize="small" />
          </ListItemIcon>
          <ListItemText>Delete</ListItemText>
        </MenuItem>
      </Menu>
    </Box>
  );
}

export default JobPipelineDisplay;
